@@ -1,6 +1,7 @@
 """Shared config dialog stuff"""
+
 # pylint: disable=not-an-iterable
-import os
+import os.path
 import shutil
 from gettext import gettext as _
 
@@ -17,9 +18,10 @@ from lutris.gui.dialogs.delegates import DialogInstallUIDelegate
 from lutris.gui.widgets.common import Label, NumberEntry, SlugEntry
 from lutris.gui.widgets.notifications import send_notification
 from lutris.gui.widgets.scaled_image import ScaledImage
-from lutris.gui.widgets.utils import MEDIA_CACHE_INVALIDATED, get_image_file_format
+from lutris.gui.widgets.utils import MEDIA_CACHE_INVALIDATED, get_image_file_extension
 from lutris.runners import import_runner
 from lutris.services.lutris import LutrisBanner, LutrisCoverart, LutrisIcon, download_lutris_media
+from lutris.services.service_media import resolve_media_path
 from lutris.util.jobs import AsyncCall
 from lutris.util.log import logger
 from lutris.util.strings import gtk_safe, parse_playtime, slugify
@@ -28,6 +30,7 @@ from lutris.util.strings import gtk_safe, parse_playtime, slugify
 # pylint: disable=too-many-instance-attributes, no-member
 class GameDialogCommon(SavableModelessDialog, DialogInstallUIDelegate):
     """Base class for config dialogs"""
+
     no_runner_label = _("Select a runner in the Game Info tab")
 
     def __init__(self, title, config_level, parent=None):
@@ -46,6 +49,7 @@ class GameDialogCommon(SavableModelessDialog, DialogInstallUIDelegate):
         self.game = None
         self.saved = None
         self.slug = None
+        self.initial_slug = None
         self.slug_entry = None
         self.directory_entry = None
         self.year_entry = None
@@ -190,20 +194,24 @@ class GameDialogCommon(SavableModelessDialog, DialogInstallUIDelegate):
         return box
 
     def _get_slug_box(self):
-        slug_box = Gtk.Box(spacing=12, margin_right=12, margin_left=12)
+        slug_box = Gtk.VBox(spacing=12, margin_right=12, margin_left=12)
 
-        label = Label(_("Identifier"))
-        slug_box.pack_start(label, False, False, 0)
+        slug_entry_box = Gtk.Box(spacing=12, margin_right=0, margin_left=0)
+        slug_label = Label()
+        slug_label.set_markup(_("Identifier\n<span size='x-small'>(Internal ID: %s)</span>") % self.game.id)
+        slug_entry_box.pack_start(slug_label, False, False, 0)
 
         self.slug_entry = SlugEntry()
         self.slug_entry.set_text(self.game.slug)
         self.slug_entry.set_sensitive(False)
         self.slug_entry.connect("activate", self.on_slug_entry_activate)
-        slug_box.pack_start(self.slug_entry, True, True, 0)
+        slug_entry_box.pack_start(self.slug_entry, True, True, 0)
 
         self.slug_change_button = Gtk.Button(_("Change"))
         self.slug_change_button.connect("clicked", self.on_slug_change_clicked)
-        slug_box.pack_start(self.slug_change_button, False, False, 0)
+        slug_entry_box.pack_start(self.slug_change_button, False, False, 0)
+
+        slug_box.pack_start(slug_entry_box, True, True, 0)
 
         return slug_box
 
@@ -338,7 +346,7 @@ class GameDialogCommon(SavableModelessDialog, DialogInstallUIDelegate):
         scale_factor = self.get_scale_factor()
         service_media = self.service_medias[image_format]
         game_slug = self.slug or (self.game.slug if self.game else "")
-        media_path = service_media.get_media_path(game_slug)
+        media_path = resolve_media_path(service_media.get_possible_media_paths(game_slug))
         image = ScaledImage.new_from_media_path(media_path, service_media.config_ui_size, scale_factor)
         image_button.set_image(image)
 
@@ -393,8 +401,9 @@ class GameDialogCommon(SavableModelessDialog, DialogInstallUIDelegate):
             self._set_image(image_type, image_button)
 
     def on_move_clicked(self, _button):
-        new_location = DirectoryDialog("Select new location for the game",
-                                       default_path=self.game.directory, parent=self)
+        new_location = DirectoryDialog(
+            "Select new location for the game", default_path=self.game.directory, parent=self
+        )
         if not new_location.folder or new_location.folder == self.game.directory:
             return
         move_dialog = dialogs.MoveDialog(self.game, new_location.folder, parent=self)
@@ -426,39 +435,42 @@ class GameDialogCommon(SavableModelessDialog, DialogInstallUIDelegate):
 
         if self.game and self.runner_name:
             self.game.runner_name = self.runner_name
-            self.game_box = self._build_options_tab(_("Game options"),
-                                                    lambda: GameBox(self.config_level, self.lutris_config, self.game),
-                                                    advanced=has_advanced(self.game),
-                                                    searchable=is_searchable(self.game))
+            self.game_box = self._build_options_tab(
+                _("Game options"),
+                lambda: GameBox(self.config_level, self.lutris_config, self.game),
+                advanced=has_advanced(self.game),
+                searchable=is_searchable(self.game),
+            )
         elif self.runner_name:
             game = Game(None)
             game.runner_name = self.runner_name
-            self.game_box = self._build_options_tab(_("Game options"),
-                                                    lambda: GameBox(self.config_level, self.lutris_config, game),
-                                                    advanced=has_advanced(game),
-                                                    searchable=is_searchable(game))
+            self.game_box = self._build_options_tab(
+                _("Game options"),
+                lambda: GameBox(self.config_level, self.lutris_config, game),
+                advanced=has_advanced(game),
+                searchable=is_searchable(game),
+            )
         else:
             self._build_missing_options_tab(self.no_runner_label, _("Game options"))
 
     def _build_runner_tab(self):
         if self.runner_name:
-            self.runner_box = self._build_options_tab(_("Runner options"),
-                                                      lambda: RunnerBox(self.config_level, self.lutris_config))
+            self.runner_box = self._build_options_tab(
+                _("Runner options"), lambda: RunnerBox(self.config_level, self.lutris_config)
+            )
         else:
             self._build_missing_options_tab(self.no_runner_label, _("Runner options"))
 
     def _build_system_tab(self):
-        self.system_box = self._build_options_tab(_("System options"),
-                                                  lambda: SystemConfigBox(self.config_level, self.lutris_config))
+        self.system_box = self._build_options_tab(
+            _("System options"), lambda: SystemConfigBox(self.config_level, self.lutris_config)
+        )
 
     def _build_options_tab(self, notebook_label, box_factory, advanced=True, searchable=True):
         if not self.lutris_config:
             raise RuntimeError("Lutris config not loaded yet")
         config_box = box_factory()
-        page_index = self._add_notebook_tab(
-            self.build_scrolled_window(config_box),
-            notebook_label
-        )
+        page_index = self._add_notebook_tab(self.build_scrolled_window(config_box), notebook_label)
 
         if page_index == 0:
             config_box.generate_widgets()
@@ -485,16 +497,13 @@ class GameDialogCommon(SavableModelessDialog, DialogInstallUIDelegate):
         self.search_entry.show_all()
 
         # Advanced settings toggle
-        switch_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL,
-                             spacing=5,
-                             no_show_all=True,
-                             visible=True)
+        switch_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5, no_show_all=True, visible=True)
         switch_box.set_tooltip_text(_("Show advanced options"))
 
         switch_label = Gtk.Label(_("Advanced"), no_show_all=True, visible=True)
         switch = Gtk.Switch(no_show_all=True, visible=True, valign=Gtk.Align.CENTER)
         switch.set_state(settings.read_setting("show_advanced_options") == "True")
-        switch.connect("state_set", lambda _w, s: self.on_show_advanced_options_toggled(bool(s)))
+        switch.connect("state-set", lambda _w, s: self.on_show_advanced_options_toggled(bool(s)))
 
         switch_box.pack_start(switch_label, False, False, 0)
         switch_box.pack_end(switch, False, False, 0)
@@ -544,12 +553,12 @@ class GameDialogCommon(SavableModelessDialog, DialogInstallUIDelegate):
             dlg = QuestionDialog(
                 {
                     "parent": self,
-                    "question":
-                        _("Are you sure you want to change the runner for this game ? "
-                          "This will reset the full configuration for this game and "
-                          "is not reversible."),
-                    "title":
-                        _("Confirm runner change"),
+                    "question": _(
+                        "Are you sure you want to change the runner for this game ? "
+                        "This will reset the full configuration for this game and "
+                        "is not reversible."
+                    ),
+                    "title": _("Confirm runner change"),
                 }
             )
 
@@ -648,7 +657,8 @@ class GameDialogCommon(SavableModelessDialog, DialogInstallUIDelegate):
 
         if not self.slug:
             self.slug = slugify(name)
-
+        if self.slug != self.initial_slug:
+            AsyncCall(download_lutris_media, None, self.slug)
         if not self.game:
             self.game = Game()
 
@@ -698,50 +708,100 @@ class GameDialogCommon(SavableModelessDialog, DialogInstallUIDelegate):
         response = dialog.run()
         if response == Gtk.ResponseType.ACCEPT:
             image_path = dialog.get_filename()
-            scale_factor = self.get_scale_factor()  # get this here for thread-safety
-            AsyncCall(self.save_custom_media, self.image_refreshed_cb, image_type, image_path, scale_factor)
+            self.save_custom_media(image_type, image_path)
         dialog.destroy()
 
     def on_custom_image_reset_clicked(self, _widget, image_type):
-        AsyncCall(self.refresh_image, self.image_refreshed_cb, image_type)
+        self.refresh_image(image_type)
 
-    def save_custom_media(self, image_type, image_path, scale_factor):
+    def save_custom_media(self, image_type, image_path):
         slug = self.slug or self.game.slug
         service_media = self.service_medias[image_type]
         self.game.custom_images.add(image_type)
-        dest_path = service_media.get_media_path(slug)
-        file_format = service_media.file_format
+        dest_paths = service_media.get_possible_media_paths(slug)
 
-        if image_path != dest_path:
-            if file_format == get_image_file_format(image_path):
-                shutil.copy(image_path, dest_path, follow_symlinks=True)
-            else:
-                # If we must transcode the image, we'll scale the image up based on
-                # the UI scale factor, to try to avoid blurriness. Of course this won't
-                # work if the user changes the scaling later, but what can you do.
-                width, height = service_media.custom_media_storage_size
-                width = width * scale_factor
-                height = height * scale_factor
-                pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(image_path, width, height)
-                # JPEG encoding looks rather better at high quality;
-                # PNG encoding just ignores this option.
-                pixbuf.savev(dest_path, file_format, ["quality"], ["100"])
+        if image_path not in dest_paths:
+            ext = get_image_file_extension(image_path)
+
+            if ext:
+                for candidate in dest_paths:
+                    if candidate.casefold().endswith(ext):
+                        self._save_copied_media_to(candidate, image_type, image_path)
+                        return
+
+            self._save_transcoded_media_to(dest_paths[0], image_type, image_path)
+
+    def _save_copied_media_to(self, dest_path, image_type, image_path):
+        """Copies a media file to the dest_path, but trashes the existing media
+        for the game first. When complete, this updates the button indicated by
+        image_type as well."""
+        slug = self.slug or self.game.slug
+        service_media = self.service_medias[image_type]
+
+        def on_trashed():
+            AsyncCall(copy_image, self.image_refreshed_cb)
+
+        def copy_image():
+            shutil.copy(image_path, dest_path, follow_symlinks=True)
             MEDIA_CACHE_INVALIDATED.fire()
-        return image_type
+            return image_type
+
+        service_media.trash_media(slug, completion_function=on_trashed)
+
+    def _save_transcoded_media_to(self, dest_path, image_type, image_path):
+        """Transcode an image, copying it to a new path and selecting the file type
+        based on the file extension of dest_path. Trashes all media for the current
+        game too. Runs in the background, and when complete updates the button indicated
+        by image_type."""
+        slug = self.slug or self.game.slug
+        service_media = self.service_medias[image_type]
+        file_format = {".jpg": "jpeg", ".png": "png"}[get_image_file_extension(dest_path)]
+
+        # If we must transcode the image, we'll scale the image up based on
+        # the UI scale factor, to try to avoid blurriness. Of course this won't
+        # work if the user changes the scaling later, but what can you do.
+        scale_factor = self.get_scale_factor()
+        width, height = service_media.custom_media_storage_size
+        width = width * scale_factor
+        height = height * scale_factor
+        temp_file = dest_path + ".tmp"
+
+        def transcode():
+            pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(image_path, width, height)
+            # JPEG encoding looks rather better at high quality;
+            # PNG encoding just ignores this option.
+            pixbuf.savev(temp_file, file_format, ["quality"], ["100"])
+            service_media.trash_media(slug, completion_function=on_trashed)
+
+        def transcode_cb(_result, error):
+            if error:
+                raise error
+
+        def on_trashed():
+            os.rename(temp_file, dest_path)
+            MEDIA_CACHE_INVALIDATED.fire()
+            self.image_refreshed_cb(image_type)
+
+        AsyncCall(transcode, transcode_cb)
 
     def refresh_image(self, image_type):
         slug = self.slug or self.game.slug
         service_media = self.service_medias[image_type]
-        dest_path = service_media.get_media_path(slug)
         self.game.custom_images.discard(image_type)
 
-        if os.path.isfile(dest_path):
-            os.remove(dest_path)
-        download_lutris_media(self.game.slug)
-        MEDIA_CACHE_INVALIDATED.fire()
+        def on_trashed():
+            AsyncCall(download, self.image_refreshed_cb)
+
+        def download():
+            download_lutris_media(slug)
+            return image_type
+
+        service_media.trash_media(slug, completion_function=on_trashed)
+
+    def refresh_image_cb(self, image_type, error):
         return image_type
 
-    def image_refreshed_cb(self, image_type, _error):
+    def image_refreshed_cb(self, image_type, _error=None):
         if image_type:
             self._set_image(image_type, self.image_buttons[image_type])
             service_media = self.service_medias[image_type]
